/*
 *
 * Program to plot a hierarchical cluster
 *
 */

#include <stdio.h>
#include <math.h>


typedef struct clust {

  char label[10];
  char name[10];
  double x;
  double y;
  int point1,point2;
} Cluster;

int NSpecies=0;
char Species[100][10];

void main(argc,argv)
int argc;
char **argv;
{
  int i,j,nl,nclust;
  Cluster *clust;
  FILE *fp;
  char file[80];
  int nm1,nm2;
  double maxdepth,x1,x2,y1,y2,y0,depth;
  char id1,id2,mem1[80],mem2[80];

  void AddSpecies();
  void *calloc();


  if(argc!=2){
    fprintf(stderr,"usage: %s cluster_file\n",argv[0]);
    exit(0);
  }

/*** First count species ***/

  fp= fopen(argv[1],"r");

  nl=0;
  while(fscanf(fp,"%*s %*s %s %s %*s %lf",mem1,mem2,&depth)!=EOF){
    if(mem1[0]=='s') AddSpecies(mem1);
    if(mem2[0]=='s') AddSpecies(mem2);
    nl++;
  }

  maxdepth=depth;

  fclose(fp);

/*** Now define base clusters ***/

  clust= (Cluster *) calloc(nl+NSpecies+1,sizeof(Cluster));

  for(i=0; i<NSpecies; i++){
    sprintf(clust[i].label,"%s",Species[i]);
    sprintf(clust[i].name,"%s",Species[i]);
    clust[i].x= (double) maxdepth*((double) i/(NSpecies-1));
    clust[i].y= 0.0;
    clust[i].point1=clust[i].point2= -1;
  }

/*** Now construct the rest of the clusters ***/

  fp= fopen(argv[1],"r");
  
  nclust= NSpecies;
  for(i=0; i<nl; i++){

    sprintf(clust[i+NSpecies].label,"c%d",i);
    fscanf(fp,"%*s %*s %s %s %*s %lf",mem1,mem2,&depth);

    for(j=0; j<nclust; j++)
      if(strcmp(clust[j].label,mem1)==0) break;
    clust[i+NSpecies].point1= j;

    for(j=0; j<nclust; j++)
      if(strcmp(clust[j].label,mem2)==0) break;
    clust[i+NSpecies].point2= j;
    
    clust[i+NSpecies].y= depth;
    clust[i+NSpecies].x= clust[clust[i+NSpecies].point1].x + 
                         (clust[clust[i+NSpecies].point2].x - clust[clust[i+NSpecies].point1].x)/2.0;


    nclust++;
  }

  fclose(fp);

/**** Now output Gnuplot File ****/


  /* First Data File */

  sprintf(file,"%s.pts",argv[1]);

  fp= fopen(file,"w");

  /* Axes */

  fprintf(fp,"%f %f\n%f %f\n\n",(double) -0.1*maxdepth/(NSpecies-1), (double) 0.0,
	                      (double) -0.1*maxdepth/(NSpecies-1) ,(double) maxdepth+maxdepth/1.8);

  fprintf(fp,"%f %f\n%f %f\n\n",(double) -0.1*maxdepth/(NSpecies-1), (double) 0.0,
	                      (double) maxdepth+0.1*maxdepth/(NSpecies-1) ,(double) 0.0);

  for(i=nclust-1; i>=NSpecies; i--){

    y0= clust[i].y;
    x1= clust[clust[i].point1].x;
    x2= clust[clust[i].point2].x;
    y1= clust[clust[i].point1].y;
    y2= clust[clust[i].point2].y;

    fprintf(fp,"%f %f\n",x1,y1);
    fprintf(fp,"%f %f\n",x1,y0);
    fprintf(fp,"%f %f\n",x2,y0);
    fprintf(fp,"%f %f\n\n",x2,y2);
  }

  fclose(fp);

  /* Now gnuplot file */

  sprintf(file,"%s.gp",argv[1]);
  fp= fopen(file,"w");

  fprintf(fp,"set term post port\nset nok\n\nse noborder\nse da s l\n");
  fprintf(fp,"se noxtics\nse noyzeroaxis\n");
  fprintf(fp,"set output '%s.ps'\n",argv[1]);
  fprintf(fp,"set title '%s'\n",argv[1]);
  fprintf(fp,"set ylabel 'depth'\n");
  
  for(i=0; i<NSpecies; i++)
    fprintf(fp,"set label %d '%s' at %f,%f\n",i+1,clust[i].name,
	    clust[i].x,-maxdepth/8);

	    /*
	    clust[i].x-maxdepth/(2*(NSpecies-1)),
	    (i%2==0 ? -maxdepth/2 : -maxdepth/4) );
	    */
  fprintf(fp,"plot '%s.pts'\n",argv[1]);
  
  fclose(fp);
}


void AddSpecies(spec)
char *spec;
{
  int i;
  
  for(i=0; i<NSpecies; i++)
    if(strcmp(spec,Species[i])==0) break;
  
  if(i==NSpecies){ 
    strcpy(Species[i],spec);
    NSpecies++;
  }


}


