/*
 *
 * Program to take a correlation file and make a list 
 * estimated causality.
 * 
 * The program dectects the maximum and the direction of tailing of
 * the lagged-correlation between two species. All correlations with
 * a given species are classified as cause, contemporary or effect
 * of the species. 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.h"
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.c"

char progid[100];

#define fcorr(a,b,c) fabs(cm[(c<nl ? nl-c-1 : c-nl)*ns*ns + (c<nl ? b : a)*ns + (c<nl ? a : b)])
#define lag(a)      (a<nl ? -(nl-a-1) : a-nl)

int main(argc,argv)
int argc;
char **argv;
{

  FILE *fp;
  int i,j,k,ns,nx,nl;
  int a,b,c,*thi,*tlo,*tmax,cause,nc;
  double cmax,thresh,*profile;
  double *cm,**dist;
  char buff[100];

  if(argc!=3){
    fprintf(stderr,"usage: %s data_file thresh\n",argv[0]);
    exit(-1);
  }

  strcpy(progid,argv[0]);

    if((fp=fopen(argv[1],"r"))==NULL){
      fprintf(stderr,"%s: Unable to open correlation file %s\n",progid,argv[1]);
      exit(-1);
    }
  
  thresh= atof(argv[2]);
  fgets(buff,81,fp);
  fscanf(fp,"%d %d %d",&nx ,&ns,&nl);
  fprintf(stdout,"%s\n%d %d %d\n",buff,nx,ns,nl);

  cm=     (double *) dvector(0,ns*ns*nl-1);
  dist=   (double **) dmatrix(0,ns-1,0,ns-1);
  for(i=0; i<ns; i++)
    for(j=0; j<ns; j++)
      dist[i][j]=1.4142135623730951;

  for(i=0; i<nl; i++){
    do{
      
      if(fscanf(fp,"%s",buff)==EOF){
	if(i==0)
	  fprintf(stderr,"%s: This file does not seem to contain any lag matrices\n",progid);
	else
	  fprintf(stderr,"%s: This file is truncated at %d when %d lags were specified.\n",progid,k,nl);
	exit(-1);
      }
    }while(strcmp(buff,"Lag")!=0);
    fscanf(fp,"%*s"); /* Get Rid of Lag # */
    for(j=0; j<ns; j++){
      for(k=0; k<ns; k++){
	fscanf(fp,"%lf",&cm[i*ns*ns+j*ns +k]);
      }
    }
  }

  fclose(fp);

  profile = (double *) calloc(2*nl,sizeof(double));
  tlo=      (int *) calloc(2*nl,sizeof(double));
  tmax=     (int *) calloc(2*nl,sizeof(double));
  thi=      (int *) calloc(2*nl,sizeof(double));

  fprintf(stderr,"Threshold= %e\n",thresh);

  for(i=0; i<ns; i++){
    fprintf(stderr,"-------------- Series #%d ------------\n",i);
    for(j=0; j<ns; j++){
      if(i==j) continue;
      for(k=0; k<2*nl; k++){
	if(k<nl) {a=j; b=i; c=nl-k-1;} 
	else 
	         {a=i; b=j; c=k-nl;}
	profile[k]= cm[c*ns*ns+a*ns + b];
      }
      nc= 0;
      for(k=0; k<2*nl; k++)
	if(fabs(profile[k])>thresh){
	  tlo[nc]= k;
	  k++;
	  cmax=0;
	  while(fabs(profile[k])>thresh){
	    if(fabs(profile[k])>cmax){
	      cmax= fabs(profile[k]);
	      tmax[nc]=k;
	    }
	    k++;
	  }
	  thi[nc]= k-1;
	  nc++;
	}


      if(nc>0){

	cmax=0.0; a=1200;
	for(k=0; k<nc; k++)
	  if(fabs(profile[tmax[k]])>cmax) {cmax= fabs(profile[tmax[k]]); a= k;}
/*
	  if(fcorr(i,j,tmax[k])>cmax) {cmax= fcorr(i,j,tmax[k]); a= k;}
*/
	fprintf(stderr,"Found %d region(s) of correlation with species %d.\n",nc,j);
	fprintf(stderr,"Region of maximum correlation (%e): %d\n",cmax,a);

	cause=0; 
	for(k=0; k<nc;k++){
	  if(lag(tmax[k])<0) cause=0;
	  if(lag(tmax[k])>0) cause=1;
	  if(lag(tmax[k])==0) cause= 2;
  fprintf(stderr,"%c Region %d: %d %d %d\t %d %s %d\n",(k==a ? '*' : ' '),k,(tlo[k]<nl ? -(nl-tlo[k]-1) : tlo[k]-nl),(tmax[k]<nl ? -(nl-tmax[k]-1 ): tmax[k]-nl),(thi[k]<nl ? -(nl-thi[k]-1) : thi[k]-nl),i,(cause == 0 ? "causes": cause== 1 ? "is caused by" : "is contemporaneous with"),j);
	 
	}
    
      }
    }
  }  
exit(0);
}










