/* Simple hierarchical clustering algorithm for quick sketches */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.h"
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.c"

extern void srand48();
extern double drand48();

int N;                     /* Number of Rows and Columns in Current Distance Matrix */
float **D1,**D2,**DT;        /* Distance Matrices*/
char **Name1,**Name2,**NT;   /* Labels for columns (rows) of Distance Matrices */

int main(argc,argv)
int argc;
char **argv;
{

  int i,j,k,l;
  float dmin;
  int lptr,nclust,c1,c2;
  int *list;
  FILE *fp;

  if(argc<2 || argc>3){
    fprintf(stderr,"usage: %s matrix_file [flag_to_print_matrices]\n",argv[0]);
    exit(-1);
  }

  if((fp=fopen(argv[1],"r"))==NULL){
    fprintf(stderr,"%s: Unable to find matrix file %s\n",argv[0],argv[1]);
    exit(-1);
  }

  fscanf(fp,"%d",&N);
  D1= (float **) matrix(0,N-1,0,N-1);
  D2= (float **) matrix(0,N-1,0,N-1);
  for(i=0; i<N; i++)
    for(j=0; j<N; j++)
      fscanf(fp,"%f",&D1[i][j]);

  fclose(fp);

  Name1= (char **) calloc(N,sizeof(char *));
  Name2= (char **) calloc(N,sizeof(char *));
  for(i=0; i<N; i++) {
    Name1[i]= (char *) calloc(6,sizeof(char));
    Name2[i]= (char *) calloc(6,sizeof(char));
    sprintf(Name1[i],"s%d",i+1);  
  }

  list= (int *) calloc(N,sizeof(int));

  nclust=0;
  while(N>1){

    dmin=100.0;

    if(argc==3)
      for(i=0; i<N; i++){
	for(j=0; j<N; j++)
	  fprintf(stdout,"%f\t",D1[i][j]);
	fprintf(stdout,"\n");
      }
    fflush(stdout);
    lptr=0;
    for(j=0; j<N; j++)
      for(i=0; i<j; i++)
	if(D1[i][j]<=dmin){
	  if(D1[i][j]<dmin) {lptr=0; dmin=D1[i][j];} 
	  list[lptr]= j*N+i;
	  lptr++;
	} 
    
    lptr--; 

    /* Randomly choose among degenerate clusters */

    if(lptr>1) lptr= (int) ((double) ((double) lptr+1.0)*drand48()); 

    c1= list[lptr]/N; c2= list[lptr]%N;
    fprintf(stdout,"Cluster %d:\t%s\t%s\t\tdepth= %e\n",nclust,Name1[c1],Name1[c2],dmin/2.0);
    fflush(stdout);

    /* Make New Matrix */

     /* First Calculated New Distances of new cluster from all others*/
     /* New cluster will be in first row */

    sprintf(Name2[0],"c%d",nclust);

    k=1;
    for(j=0; j<N; j++){
      if(j==c1 || j==c2) continue;
      
      /*        D2[0][k]=D2[k][0]=0.5*(D1[c1][j]+D1[c2][j]); */
      D2[0][k]=D2[k][0]= (D1[c1][j]<D1[c2][j] ? D1[c1][j] : D1[c2][j]);
      l=1;
      strcpy(Name2[k],Name1[j]);
      for(i=0; i<=j; i++)
	if(i==c1 || i==c2) continue;
	else{
	  D2[l][k]=D2[k][l]=D1[i][j];
	  l++;
	}
      k++;
    }
    
    N--;
    nclust++;

    DT=D1;    NT= Name1;
    D1=D2;    Name1= Name2;
    D2=DT;    Name2= NT;
  }

exit(0);
}


