/*
 *
 * Program to take a correlation file and make the minimum totally connected
 * network from the correlation surface.
 *
 * Modified May 26, 1994 to only use tau=0 correlation and output gnuplot input for lines.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.h"
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.c"

char progid[100];
double *cm;
int ns,nl;

int main(argc,argv)
int argc;
char **argv;
{

  FILE *fp;
  int i,j,k,nx,ng,dim;
  double r,rmax;
  int **Group,*NGroup,*group,*connt1,*connt2;
  double **x;
  int njoin,s1,s2;
  char buff[3000];
  void JoinGroups();
  void FindMax();
  char file[80];

  if(argc!=4){
    fprintf(stderr,"usage: %s prefix pts_postfix dimension\n",argv[0]);
    exit(-1);
  }

  strcpy(progid,argv[0]);
  dim= atoi(argv[3]);
 


  sprintf(file,"%s.sp.cor2",argv[1]);

    if((fp=fopen(file,"r"))==NULL){
      fprintf(stderr,"%s: Unable to open correlation file %s\n",progid,file);
      exit(-1);
    }
  
  fgets(buff,81,fp);
  fscanf(fp,"%d %d %d",&nx ,&ns,&nl);

  cm=     (double *) dvector(0,ns*ns*nl-1);

  for(i=0; i<nl; i++){
    do{
      
      if(fscanf(fp,"%s",buff)==EOF){
	if(i==0)
	  fprintf(stderr,"%s: This file does not seem to contain any lag matrices\n",progid);
	else
	  fprintf(stderr,"%s: This file is truncated at %d when %d lags were specified.\n",progid,k,nl);
	exit(-1);
      }
    }while(strcmp(buff,"Lag")!=0);
    fscanf(fp,"%*s"); /* Get Rid of Lag # */
    for(j=0; j<ns; j++){
      for(k=0; k<ns; k++){
	fscanf(fp,"%lf",&cm[i*ns*ns+j*ns +k]);
      }
    }
  }

  fclose(fp);

/* Get Coordinates */

  x = dmatrix(0,ns,0,dim);
  sprintf(file,"%s.sp.%s",argv[1],argv[2]);

    if((fp=fopen(file,"r"))==NULL){
      fprintf(stderr,"%s: Unable to open distance file %s\n",progid,file);
      exit(-1);
    }

  for(i=0; i<ns; i++){
    for(j=0; j<dim; j++)
      fscanf(fp,"%lf",&x[i][j]);

    fgets(buff,2999,fp);
  }

  fclose(fp);

/* 
 * This algorithm takes a number of passes until
 *
 * 0) Put each series into its own group.
 * 1) Connect a group to another group by finding the maximal correlation between a series in G1 with one in G2 
 * 2) Store combined group as a single group and decrement group number 
 * 2) Count How many Disjoint groups there are.
 * 3) If there is only one group then quit.
 * 4) else goto 1
 */

  /* Initialize Groups */

  Group=  (int **) imatrix(0,ns-1,0,ns-1);
  NGroup= (int *)  ivector(0,ns-1);
  group=  (int *)  ivector(0,ns-1);
  connt1= (int *)  ivector(0,ns-1);
  connt2= (int *)  ivector(0,ns-1);

  for(i=0; i<ns; i++){
    Group[i][0]=i;
    NGroup[i]=1;
  }
  ng=ns;

  do{
    for(i=0; i<ng; i++){
      rmax= -2.0; njoin=0;
      for(j=0; j<ng; j++){
	if(j==i) continue;
	r=0.0;
	FindMax(i,j,Group,NGroup,&r,&s1,&s2);
	if(r>=rmax){

	  if(r>rmax) {rmax=r; njoin=1;}
	  else if((rmax-r)/rmax<=0.05) njoin++;
  
	  group[njoin-1]=j;
	  connt1[njoin-1]=s1;
	  connt2[njoin-1]=s2;
	}
      }

      for(j=0; j<njoin; j++){
	JoinGroups(i,j,njoin,group,Group,NGroup,&ng);
	
	for(k=0; k<dim; k++) fprintf(stdout,"%e\t",x[connt1[j]][k]);
	fprintf(stdout,"\n");
	for(k=0; k<dim; k++) fprintf(stdout,"%e\t",x[connt2[j]][k]);
	fprintf(stdout,"\n\n");
	fprintf(stderr,"Connect %d to %d\n",connt1[j]+1,connt2[j]+1);
	fflush(stderr); fflush(stdout);
      }
    }
  } while(ng>1);

exit(0);
}

void FindMax(n,m,G,N,rmax,s1,s2)
int n,m,*s1,*s2;
int **G;
int *N;
double *rmax;
{
  register int u,v,k;
  int i,j,a,b,c;
  double r;

  for(u=0; u<N[n]; u++){
    for(v=0; v<N[m]; v++){
      i=G[n][u];
      j=G[m][v];
      c=0; a=i; b=j;

      r= fabs(cm[c*ns*ns+a*ns+b]);
      if(r>*rmax){
	/******* Later make sure I include the equal sign and make a list **********/
	*rmax=r;
	*s1= (a>b ? b : a);
	*s2= (a>b ? a : b);
      }

/*
      for(k=0; k<2*nl; k++){ 
	if(k<nl) {a=j; b=i; c=nl-k-1;} 
	else 
	  {a=i; b=j; c=k-nl;}
	  r= fabs(cm[c*ns*ns+a*ns+b]);
	  if(r>*rmax){
	    *rmax=r;
	    *s1= (a>b ? b : a);
	    *s2= (a>b ? a : b);
	  }
      }
*/

    }
  }
}

void JoinGroups(g1,g2,n,g,G,N,ng)
int g1,g2,*g,**G,*N,*ng,n;
{
  int i,j;
  register int k;

  i= (g1<g[g2] ? g1 : g[g2]);  j= (g1<g[g2] ? g[g2] : g1);

  for(k=0; k<N[j]; k++)
    G[i][N[i]+k]=G[j][k];

  N[i] += N[j];

  for(k=j+1; k< *ng; k++){
    memcpy(G[k-1],G[k],N[k]*sizeof(int));
    N[k-1]=N[k];
  }

  for(k=0; k<n; k++)
    if(g[k]>g[g2]) g[k]--;
  *ng -= 1;
}











