/*
 *
 * Program to take a correlation file and make a list of all connections
 * between series with a correlation greater than 'thresh'. 
 * Output is of this form:
 *
 * Series #n:
 *
 *             Series#1 [interval 1] [interval 2]......[interval m]
 *             etc.
 *
 * where [interval] is in the form [lo,max,hi] where low, and high demark
 * two lags between which the correlation with series n is above thresh and
 * max is the maximum within the interval.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.h"
#include "/usr/local/numerics/recipes_c-kr/misc/nrutil.c"

char progid[100];

int main(argc,argv)
int argc;
char **argv;
{

  FILE *fp;
  int i,j,k,ns,nx,nl;
  int a,b,c,first,maxl,high,flag;
  double d,maxc,ctmp,thresh;
  double *cm,**dist;
  char buff[100];

  if(argc!=3){
    fprintf(stderr,"usage: %s data_file thresh\n",argv[0]);
    exit(-1);
  }

  strcpy(progid,argv[0]);

    if((fp=fopen(argv[1],"r"))==NULL){
      fprintf(stderr,"%s: Unable to open correlation file %s\n",progid,argv[1]);
      exit(-1);
    }
  
  thresh= atof(argv[2]);
  fgets(buff,81,fp);
  fscanf(fp,"%d %d %d",&nx ,&ns,&nl);
  fprintf(stdout,"%s\n%d %d %d\n",buff,nx,ns,nl);

  cm=     (double *) dvector(0,ns*ns*nl-1);
  dist=   (double **) dmatrix(0,ns-1,0,ns-1);
  for(i=0; i<ns; i++)
    for(j=0; j<ns; j++)
      dist[i][j]=1.4142135623730951;

  for(i=0; i<nl; i++){
    do{
      
      if(fscanf(fp,"%s",buff)==EOF){
	if(i==0)
	  fprintf(stderr,"%s: This file does not seem to contain any lag matrices\n",progid);
	else
	  fprintf(stderr,"%s: This file is truncated at %d when %d lags were specified.\n",progid,k,nl);
	exit(-1);
      }
    }while(strcmp(buff,"Lag")!=0);
    fscanf(fp,"%*s"); /* Get Rid of Lag # */
    for(j=0; j<ns; j++){
      for(k=0; k<ns; k++){
	fscanf(fp,"%lf",&cm[i*ns*ns+j*ns +k]);
      }
    }
  }

  fclose(fp);

  for(i=0; i<ns; i++){
    fprintf(stdout,"Series #%d:\n",i);
    for(j=0; j<ns; j++){
      first=0;
      d=1.4142135623730951;
      for(k=0; k<2*nl; k++){
	if(k<nl) {a=j; b=i; c=nl-k-1;} 
	else 
	         {a=i; b=j; c=k-nl;}
	if(fabs(ctmp=cm[c*ns*ns+a*ns + b])>=thresh){
	  if(first==0){ first=1; fprintf(stdout,"\t%d: ",j);}
	  fprintf(stdout,"[%+2d,",(k<nl ? -c : c));
	  maxc=0.0; maxl=high=(k<nl ? -c : c);
	  while((ctmp=fabs(cm[c*ns*ns+a*ns + b]))>=thresh){
	    if(ctmp>maxc){ 
	      fprintf(stderr,"r[%d,%d]: omax= %e nmax= %e d= %e nd= %e\n",b,a,maxc,ctmp,d,1.41421*sqrt(1.0-fabs(ctmp)));
	      maxc=ctmp; 
	      maxl=(k<nl ? -c : c);
	      d= 1.4142135623730951*sqrt(1.0-fabs(maxc));
/*
	      d= 0.5*((1.0-maxc)+((double) abs(maxl)/nl));
*/
	    }
	    high=(k<nl ? -c : c);
	    k++;
	    if(k==2*nl) break;
	    if(k<nl) {a=j; b=i; c=nl-k-1;} else {a=i; b=j; c=k-nl;}
	  }
	  fprintf(stdout,"%+2d,%+2d](%.3e)\t",maxl,high,maxc);
	  if(d<dist[i][j]) dist[i][j]=d;
	}
      }
	if(first==1) fprintf(stdout,"\n");
    }
  }

  /* Determine if the correlation space is a metric space */

  fprintf(stdout,"\n\n**** Metric Space Errors ****\n\n");
  for(i=0; i<ns; i++){
    flag=0;
    for(j=0; j<ns; j++)
      if(dist[i][j]<0.0) {flag=1; break;}
    if(flag==1) fprintf(stdout,"Matrix Failed Positivity at (%d,%d)\n",i,j);
  }
 
  for(i=0; i<ns; i++){
    flag=0;
    for(j=0; j<ns; j++)
      if(dist[i][j]==0.0 && i!=j) {flag=1; break;}
    if(flag==1) {
      dist[i][j]= 1e-10;
      fprintf(stdout,"Matrix Failed Identity Test at (%d,%d)\n",i,j);
    }
  }

  for(i=0; i<ns; i++){
    flag=0;
    for(j=0; j<ns; j++)
      if(dist[i][j]!=dist[j][i]) {flag=1; break;}
    if(flag==1) fprintf(stdout,"Matrix Failed Symmetry Test at (%d,%d)\n",i,j);
  }

  for(i=0; i<ns; i++){
    for(j=0; j<ns; j++){
      flag=0;
      for(k=0; k<ns; k++)
      if(dist[i][k]>dist[i][j]+dist[j][k]) {flag=1; break;}
      if(flag==1) fprintf(stdout,"Matrix Failed Triangle Test at (%d,%d,%d)\n",i,j,k);  
    }
  }

/*
  for(i=0; i<ns; i++){
    for(j=0; j<ns; j++){
      flag=0;
      for(k=0; k<ns; k++)
      if(dist[i][k]>(dist[i][j]>dist[j][k] ? dist[i][j] : dist[j][k])) {flag=1; break;}
      if(flag==1) fprintf(stdout,"Matrix Failed Ultrametricity Test at (%d,%d,%d)\n",i,j,k);  
    }
  }
*/

/*
  for(i=0; i<ns; i++){
    for(j=0; j<ns; j++){
      for(k=0; k<ns; k++){
	flag=0;
	for(h=0; h<ns; h++){
	  maxd= ((dist[h][j]+dist[i][k]) > (dist[h][k] + dist[i][j]) ? (dist[h][j]+dist[i][k]): (dist[h][k]+dist[i][j]));
	  if((dist[h][i]+dist[j][k]) > maxd) 
	    {flag=1; break;}
	}
      if(flag==1) fprintf(stdout,"Matrix Failed Additivity Test at (%d,%d,%d)\n",i,j,k);  
      }
    }
  }
*/
  fprintf(stdout,"\n\n*****************************\n\n");

  fprintf(stdout,"%d\n",ns);
  for(i=0; i<ns; i++){
    for(j=0; j<ns; j++)
      fprintf(stdout,"%.6e ",dist[i][j]);
    fprintf(stdout,"\n");
  }
     
exit(0);
}










